/*
   getch(), getche(), kbhit() function Ver. 1.0
       Programmed by Kenzi NOIKE
*/


#include  <stdio.h>
#include  <string.h>
#include  <unistd.h>
#include  <ctype.h>
#include  <sys/ioctl.h>
#include  <sys/time.h>
#include  <sys/types.h>

#ifdef FBSD
#include  <sys/tty.h>
#endif

#if defined(NEWS)
static struct sgttyb org_tbuf;
static struct sgttyb set_tbuf;
#elif (defined(FBSD) || defined(LINUX) )
#include  <termios.h>
static struct termios org_tbuf;
static struct termios set_tbuf;
#endif

#include  "getchkbh.h"

static int call_flag;

extern "C" {

int init_getch_kbhit(void)
{
	if(call_flag != 0){
		return 0;
	}
	else{
		call_flag = 1;
	}

#if defined(NEWS)

	if(ioctl(0, TIOCGETP, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}
	if(ioctl(0, TIOCGETP, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}
	
	set_tbuf.sg_flags = (set_tbuf.sg_flags | RAW);
	set_tbuf.sg_flags = (set_tbuf.sg_flags & ~ECHO_X);
	
	if(ioctl(0, TIOCSETP, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	return 1;


#elif defined(LINUX)

	if(tcgetattr(0, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	if(tcgetattr(0, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	set_tbuf.c_lflag = (set_tbuf.c_lflag & (~ICANON));
	set_tbuf.c_lflag = (set_tbuf.c_lflag & (~ECHO));

	if(tcsetattr(0, TCSAFLUSH, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}
	
	return 1;


#elif defined(FBSD)

	if(ioctl(0, TIOCGETA, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}
	if(ioctl(0, TIOCGETA, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	set_tbuf.c_lflag = (set_tbuf.c_lflag & ~ICANON);

	if(ioctl(0, TIOCSETA, &set_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	return 1;


#else

	return -1;

#endif

}


int getch(void)
{
	int retval;

	if(call_flag == 0){
		init_getch_kbhit();
	}
	
	if((tolower(retval = getc(stdin))) == 'q'){
		close_getch_kbhit();
	}

	return retval;
}


int getche(void)
{
	int buf;
	
	if(call_flag == 0){
		init_getch_kbhit();
		call_flag = 1;
	}

	buf = getc(stdin);
	putc(buf, stdout);
	
	return buf;
}


int kbhit(void)
{
	fd_set readfds;
	struct timeval timeout;
	
	fflush(stdout);  /*  Ƃ肠  */

	if(call_flag == 0){
		init_getch_kbhit();
		call_flag = 1;
	}

	FD_ZERO(&readfds);
	FD_SET(0, &readfds);
	timeout.tv_sec = 0;
	timeout.tv_usec = 10;
	select(1, &readfds, NULL, NULL, &timeout);
	if(FD_ISSET(0, &readfds)){
		return (!0);
	}
	else{
		return 0;
	}
}


int close_getch_kbhit(void)
{
	if(call_flag == 0){
		return 0;
	}
	else{
		call_flag = 0;
	}
	
#if defined(NEWS)

	if(ioctl(0, TIOCSETP, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}

	return 1;


#elif defined(LINUX)

	if(tcsetattr(0, TCSAFLUSH, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}


	return 1;


#elif defined(FBSD)

	if(ioctl(0, TIOCSETA, &org_tbuf) == -1){
		fprintf(stderr, "ioctl error!\n");
		exit(0);
	}
	
	return 1;


#else	
	
	return -1;

#endif

}
}
